import bpy
import os
import importlib
from . import bl_info, main_fluid_version
from .FP_Utilities import compare_versions


other_addon_name = ""


def set_addon_settings(addon_name, fluid_version):
    global other_addon_name
    global other_fluid_version
    other_addon_name = addon_name
    other_fluid_version = fluid_version


def add_asset_library(asset_library_name, asset_library_path):
    preferences = bpy.context.preferences
    asset_libraries = preferences.filepaths.asset_libraries

    # Check if a library with the same name in path exists and remove it if found
    for i, library in enumerate(asset_libraries):
        if asset_library_name in library.path:
            bpy.ops.preferences.asset_library_remove(index=i)
            break

    # Add the library
    bpy.ops.preferences.asset_library_add()
    new_library = bpy.context.preferences.filepaths.asset_libraries[-1]
    new_library.name = asset_library_name
    new_library.path = asset_library_path

    # Check the Blender version and set the import method accordingly
    if bpy.app.version >= (3, 5, 0):
        new_library.import_method = "APPEND"


def remove_asset_library(asset_library_name):
    preferences = bpy.context.preferences
    asset_libraries = preferences.filepaths.asset_libraries

    for i, library in enumerate(asset_libraries):
        if library.name == asset_library_name:
            bpy.ops.preferences.asset_library_remove(index=i)
            break


def library_exists(asset_library_name, asset_library_path):
    preferences = bpy.context.preferences
    asset_libraries = preferences.filepaths.asset_libraries

    for library in asset_libraries:
        if library.name == asset_library_name and library.path == asset_library_path:
            return True

    return False


def clean_asset_libraries(asset_library_name, asset_library_path):
    preferences = bpy.context.preferences
    asset_libraries = preferences.filepaths.asset_libraries

    for library in asset_libraries:

        #Clean extra libraries that have suffix like .001, .002 etc
        if library.name.startswith(asset_library_name):
            suffix = library.name[len(asset_library_name):].lstrip('.')
            if suffix.isdigit():
                remove_asset_library(library.name)
                
        #Check Blender Version
        if library.name == asset_library_name and library.path != asset_library_path:
            remove_asset_library(asset_library_name)


def update_asset_browser(self, context, asset_library_name, asset_library_path):
    if other_addon_name in bpy.context.preferences.addons.keys():
        addon_module = importlib.import_module(other_addon_name)
        addon_module.update_asset_browser_proxy()

    if self.add_asset_browser:
        if not library_exists(asset_library_name, asset_library_path):
            add_asset_library(asset_library_name, asset_library_path)
    else:
        remove_asset_library(asset_library_name)

     
class FluidPainterSettingsPanel(bpy.types.Panel):
    bl_label = "Settings"
    bl_idname = "OBJECT_PT_fluid_settings"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "Fluid Painter"
    bl_options = {"DEFAULT_CLOSED"}

    def draw(self, context):
        layout = self.layout

        preferences = bpy.context.preferences
        asset_libraries = preferences.filepaths.asset_libraries
        
        addon_prefs = context.preferences.addons[__package__].preferences
        
        layout.label(text=bl_info["name"] + " Version: {}.{}.{}".format(*bl_info["version"]))
        
        if other_addon_name in bpy.context.preferences.addons.keys():
            other_addon_module = importlib.import_module(other_addon_name)  # Import the other addon's module dynamically
            other_addon_info = other_addon_module.bl_info  # Access its bl_info attribute
            other_addon_label = other_addon_info.get('name')
            other_addon_version = other_addon_info.get('version', (0, 0, 0))  # Get version, or use (0, 0, 0) as a default
            formatted_version = "{}.{}.{}".format(*other_addon_version)  # Format the version
            
            outdated_text = " (Outdated)" if compare_versions(other_fluid_version, main_fluid_version) < 0 else ""
            
            layout.label(text=f"{other_addon_label} Version: {formatted_version}" + outdated_text)
            
        layout.label(text=bl_info["name"] + " GN Version: " + main_fluid_version)

        box = layout.box()
        row = box.row(align=True)
        row.operator("wm.url_open", text="Documentation", icon='URL').url = "https://drive.google.com/drive/folders/1YN6vM20hvLPCkmahKYUeMqXxvjgE8Wyd?usp=drive_link"
        row.operator("wm.url_open", text="YouTube Tutorial", icon='URL').url = "https://youtube.com/playlist?list=PLk8Hvw7zTT48rB1RjO8eGV23zjP4a-KKg"
        
        box = layout.box()
        box.label(text="Fluid Presets")
        box.prop(addon_prefs, "add_fluid_panel", text="Show in Fluid Add Panel")
        box.prop(addon_prefs, "add_asset_browser", text="Show in Asset Browser")

        box = layout.box()
        box.label(text="Fluid Creation")
        box.prop(addon_prefs, "edit_on_add")