from .download import download_zip, download_file
from pathlib import Path
down_path = Path(__file__).parent
xz_url = "addons_file"+"/"+down_path.name+"/"
download_zip(xz_url + "Library42.zip",down_path)
download_zip(xz_url + "Pics.zip",down_path)
download_zip(xz_url + "Tools.zip",down_path)


bl_info = {
    "name": "Fluid Painter",
    "author": "Miki3d",
    "version": (1, 3, 21),
    "blender": (3, 1, 0),
    "location": "View3D > Tools",
    "description": "Fluid Painter Tool",
    "doc_url": "https://drive.google.com/drive/folders/1YN6vM20hvLPCkmahKYUeMqXxvjgE8Wyd?usp=drive_link",
    "category": "Paint",
}


import bpy
import os
from bpy.types import AddonPreferences
from bpy.utils import previews
from bpy.app.handlers import persistent

main_fluid_version = "1.3.12"


# Specify the name and path of the asset library
asset_library_name = "Fluid Painter Library"
if bpy.app.version >= (4, 2, 0):
    asset_library_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "Library42")
else:
    asset_library_path = os.path.join(os.path.dirname(os.path.realpath(__file__)), "Library")


from .FP_Utilities import (
    debug_print,
)


from .FP_Settings import (
    add_asset_library,
    remove_asset_library,
    library_exists,
    update_asset_browser,
    clean_asset_libraries,
    FluidPainterSettingsPanel,
)


from .FP_Append import (
    FluidPainterProperties,
    AppendOperator,
    FluidPainterAppendPanel,
    update_image_preview,
    update_image_preview_undo,
    add_fluid_from_asset_browser
)


from .FP_Edit import (
    StartDrawingOperator,
    StopDrawingOperator,
    SetDrawingObjectsOperator,
    SetFrameDriverOperator,
    SetSingleStreamOperator,
    FluidPainterEditingPanelMain,
    FluidPainterEditingPanelProperties,
    FluidPainterEditingPanelMaterial,
    FluidPainterEditingPanelDrop,
    FluidPainterEditingPanelLayerA,
    FluidPainterEditingPanelLayerB,
)


from .FP_Tools import (
    ConvertToMeshOperator,
    SetOriginOperator,
    VexterGroupsTransfer,
    BindToMesh,
    MergeFluids,
    UnmergeFluids,
    SwitchRestPose,
    ConvertToFluidOperator,
    FluidPainterToolsPanelMain,
    FluidPainterToolsPanelSetOrigin,
    FluidPainterToolsPanelConvertToMesh,
    FluidPainterToolsPanelConvertToFluid,
    FluidPainterToolsPanelStickToMesh,
    FluidPainterToolsPanelMergeFluids,
)


from .FP_Update import (
    UpdateCurveOperator,
    FluidPainterUpdatePanel,
)
from .translation.translate import GetTranslationDict


class FluidPainterPreferences(AddonPreferences):
    # This must match the add-on name, use '__package__'
    # when defining this in a submodule of a Python package.
    bl_idname = __package__

    edit_on_add: bpy.props.BoolProperty(
        name="Draw on Add",
        default=True,
        description="Enable drawing on object creation",
    )
    
    # Add the add_fluid_enabled flag
    add_fluid_panel: bpy.props.BoolProperty(
        default=True,
        description="Enable/disable the Fluid Add Panel",
    )
    
    # Add the add_fluid_enabled flag
    add_asset_browser: bpy.props.BoolProperty(
        default=True,
        description="Enable/disable the Asset Browser",
        update=lambda self, context: update_asset_browser(self, context, asset_library_name, asset_library_path)  
    )
    
    # Property for fluid scale
    fluid_scale: bpy.props.FloatProperty(
        name="Fluid Scale",
        description="Scale factor for the fluid",
        default=1.0,
        min=0.1,
        max=100000.0,
    )
    
    # Define a property for spline resolution
    spline_resolution: bpy.props.IntProperty(
        name="Spline Resolution",
        description="Resolution for the spline",
        default=12,
        min=2,
        soft_max=64,
    )

    fluid_is_topping: bpy.props.BoolProperty(
        name="Fluid is Topping",
        description="The fluid is used as topping",
        default=False,
    )
    
    parent_fluid: bpy.props.BoolProperty(
        name="Parent Fluid",
        description="Parent Fluid to selected Object",
        default=False,
    )


# Call the update_image_preview function in a loop until the scene is ready
def scene_checker():
    if bpy.context.scene is not None:
    
        clean_asset_libraries(asset_library_name, asset_library_path)
    
        update_image_preview(bpy.context)
        debug_print("Image preview updated by Addon Install")
        
        preferences = bpy.context.preferences
        addon_prefs = preferences.addons[__package__].preferences
        
        if addon_prefs.add_asset_browser:
            if not library_exists(asset_library_name, asset_library_path):
                add_asset_library(asset_library_name, asset_library_path)
            
        return None  # Return None to stop the timer
        
    return 1.0  # Check again in 1 second


@persistent
def load_handler(dummy):
    update_image_preview(bpy.context)
    debug_print("Image preview updated by Load Handler")


classes = [
    FluidPainterSettingsPanel,
    FluidPainterPreferences,
    FluidPainterProperties,
    AppendOperator,
    FluidPainterAppendPanel,
    StartDrawingOperator,
    StopDrawingOperator,
    FluidPainterEditingPanelMain,
    FluidPainterEditingPanelProperties,
    FluidPainterEditingPanelMaterial,
    FluidPainterEditingPanelDrop,
    FluidPainterEditingPanelLayerA,
    FluidPainterEditingPanelLayerB,
    SwitchRestPose,
    VexterGroupsTransfer,
    BindToMesh,
    MergeFluids,
    UnmergeFluids,
    ConvertToFluidOperator,
    ConvertToMeshOperator,
    SetOriginOperator,
    SetDrawingObjectsOperator,
    SetFrameDriverOperator,
    SetSingleStreamOperator,
    FluidPainterToolsPanelMain,
    FluidPainterToolsPanelSetOrigin,
    FluidPainterToolsPanelConvertToMesh,
    FluidPainterToolsPanelConvertToFluid,
    FluidPainterToolsPanelStickToMesh,
    FluidPainterToolsPanelMergeFluids,
    FluidPainterUpdatePanel,
    UpdateCurveOperator,
]


def register():

    for cls in classes:
        try:
            bpy.utils.register_class(cls)
            debug_print(f'Registered {cls.__name__}')
        except Exception as e:
            debug_print(f'Error registering {cls.__name__}: {e}')
        
    bpy.types.Scene.fluid_painter = bpy.props.PointerProperty(type=FluidPainterProperties)
    
    bpy.app.timers.register(scene_checker)
    
    bpy.app.handlers.load_post.append(load_handler)
    bpy.app.handlers.depsgraph_update_pre.append(add_fluid_from_asset_browser)
    bpy.app.handlers.undo_post.append(update_image_preview_undo)
    try:
        bpy.app.translations.register(__package__, GetTranslationDict())
    except:
        pass

def unregister():
    for cls in classes:
        try:
            bpy.utils.unregister_class(cls)
            debug_print(f'Unregistered {cls.__name__}')
        except Exception as e:
            debug_print(f'Error unregistering {cls.__name__}: {e}')
            
    if FP_Append.pcoll:
        previews.remove(FP_Append.pcoll)
        debug_print("Preview removed")
    FP_Append.pcoll = None
    
    remove_asset_library(asset_library_name)
        
    del bpy.types.Scene.fluid_painter
    
    bpy.app.handlers.load_post.remove(load_handler)
    bpy.app.handlers.depsgraph_update_pre.remove(add_fluid_from_asset_browser)
    bpy.app.handlers.undo_post.remove(update_image_preview_undo)
    try:
        bpy.app.translations.unregister(__package__)
    except:
        pass
      
    
if __name__ == "__main__":
    register()
